//
// Simple program to draw 3d surfaces defined on rectangular mesh
//
#include <math.h>
#include <iostream>
#include <SDL2/SDL.h> // SDL2.0 library

SDL_Window* window = NULL;
SDL_Renderer* renderer = NULL;

// Prototypes:
double u(double x, double y); // Function to plot
void w_plot(double **x, double **y, double **z, int nx, int ny, double x0, double xsc, double y0, double ysc, double zsc, uint col); // Draw plot
void w_axes(); // Draw axis
void w_clear(); // Clear screen

// Main function
int main(int argc, char* argv[]) {

  // Set numbers of steps and values of steps
  int nx = 10;
  int ny = 10;
  std::cout << "nx = ";
  std::cin >> nx;
  std::cout << "ny = ";
  std::cin >> ny;
  double hx = 5.5/nx;
  double hy = 1./ny;

  // Initialize graphics
  SDL_Event event;
  SDL_CreateWindowAndRenderer(1024, 768, 0, &window, &renderer);
  w_clear();

  // Allocate arrayes
  double **u_x = new double*[nx+1];
  double **u_y = new double*[nx+1];
  double **u_z = new double*[nx+1];
  int i, j;
  for (i = 0; i <= nx; i++) {
    u_x[i] = new double[ny+1];
    u_y[i] = new double[ny+1];
    u_z[i] = new double[ny+1];
  }

  // Set the 2d mesh and calculate function u
  for (i = 0; i <= nx; i++) {
    for (j = 0; j <= ny; j++) {
      u_x[i][j] = .5 + i*hx;
      u_y[i][j] = j*hy;
      u_z[i][j] = u(u_x[i][j], u_y[i][j]);
    }
  }

  // Draw plots
  w_axes();
  w_plot(u_x, u_y, u_z, nx+1, ny+1, .5, 5.5, 0., 1., 2., 5);

  // Quit
  std::cin.ignore(); std::cin.ignore();

  // Deallocate arrayes
  for (i = 0; i <= nx; i++) {
    delete[] u_x[i];
    delete[] u_y[i];
    delete[] u_z[i];
  }
  delete u_x;
  delete u_y;
  delete u_z;

  // Close and destroy the window and renderer
  SDL_DestroyRenderer(renderer);
  SDL_DestroyWindow(window);

  SDL_Quit(); // Clean up
  return 0; // Return
}


// Function to plot
double u(double x, double y) {
  return sin(x)*cos(y) + 1.;
}

// Draw plot
// x, y, z are arrayes of node coordinates
// nx, ny are sizes of arrayes
// x0, y0 are coordinates of zero point
// xsc, ysc, zsc are scales on the x, y, z axis
// col is a color of the plot
void w_plot(double **x, double **y, double **z, int nx, int ny, double x0, double xsc, double y0, double ysc, double zsc, uint col) {
  if (col == 0) { // Choose color for plot
    SDL_SetRenderDrawColor(renderer, 255, 0, 0, SDL_ALPHA_OPAQUE); // red
  } else if (col == 1) {
    SDL_SetRenderDrawColor(renderer, 0, 255, 0, SDL_ALPHA_OPAQUE); // green
  } else if (col == 2) {
    SDL_SetRenderDrawColor(renderer, 0, 0, 255, SDL_ALPHA_OPAQUE); // blue
  } else if (col == 3) {
    SDL_SetRenderDrawColor(renderer, 255, 255, 0, SDL_ALPHA_OPAQUE); // yellow
  } else if (col == 4) {
    SDL_SetRenderDrawColor(renderer, 255, 0, 255, SDL_ALPHA_OPAQUE); // magenta
  } else {
    SDL_SetRenderDrawColor(renderer, 0, 255, 255, SDL_ALPHA_OPAQUE); // cyan
  }

  // Set steps
  int si = nx/20 + 1; // Plot 10-20 lines
  int sj = ny/20 + 1;

  // Set usefull geometric constants
  double c60 = .5*344;
  double s60 = .866*344;

  double xa, xb;
  double ya, yb;
  int i, j; // Main drawing loops
  for (i = 0; i < nx; i=i+si) {
    for (j = 1; j < ny; j++) {
      xa = (x[i][j-1] - x0)/xsc;
      xb = (x[i][j] - x0)/xsc;
      ya = (y[i][j-1] - y0)/ysc;
      yb = (y[i][j] - y0)/ysc;
      SDL_RenderDrawLine(renderer, 512 - xa*s60 + ya*s60, 384 + xa*c60 + ya*c60 - z[i][j-1]*344/zsc, \
                                   512 - xb*s60 + yb*s60, 384 + xb*c60 + yb*c60 - z[i][j]*344/zsc);
    }
  }
  for (j = 1; j < ny; j++) {
    xa = (x[nx-1][j-1] - x0)/xsc;
    xb = (x[nx-1][j] - x0)/xsc;
    ya = (y[nx-1][j-1] - y0)/ysc;
    yb = (y[nx-1][j] - y0)/ysc;
    SDL_RenderDrawLine(renderer, 512 - xa*s60 + ya*s60, 384 + xa*c60 + ya*c60 - z[nx-1][j-1]*344/zsc, \
                                 512 - xb*s60 + yb*s60, 384 + xb*c60 + yb*c60 - z[nx-1][j]*344/zsc);
  }
  for (j = 0; j < ny; j=j+sj) {
    for (i = 1; i < nx; i++) {
      xa = (x[i-1][j] - x0)/xsc;
      xb = (x[i][j] - x0)/xsc;
      ya = (y[i-1][j] - y0)/ysc;
      yb = (y[i][j] - y0)/ysc;
      SDL_RenderDrawLine(renderer, 512 - xa*s60 + ya*s60, 384 + xa*c60 + ya*c60 - z[i-1][j]*344/zsc, \
                                   512 - xb*s60 + yb*s60, 384 + xb*c60 + yb*c60 - z[i][j]*344/zsc);
    }
  }
  for (i = 1; i < nx; i++) {
    xa = (x[i-1][ny-1] - x0)/xsc;
    xb = (x[i][ny-1] - x0)/xsc;
    ya = (y[i-1][ny-1] - y0)/ysc;
    yb = (y[i][ny-1] - y0)/ysc;
    SDL_RenderDrawLine(renderer, 512 - xa*s60 + ya*s60, 384 + xa*c60 + ya*c60 - z[i-1][ny-1]*344/zsc, \
                                 512 - xb*s60 + yb*s60, 384 + xb*c60 + yb*c60 - z[i][ny-1]*344/zsc);
  }
  SDL_RenderPresent(renderer);
}

// Draw axis
void w_axes() {
  SDL_SetRenderDrawColor(renderer, 255, 255, 255, SDL_ALPHA_OPAQUE); // white
  SDL_RenderDrawLine(renderer, 512, 384, 512, 20); // Draw z axis
  SDL_RenderDrawLine(renderer, 512, 20, 508, 28);
  SDL_RenderDrawLine(renderer, 512, 20, 516, 28);
  SDL_RenderDrawLine(renderer, 508, 40, 516, 40);  // one tick H = 344
  SDL_RenderDrawLine(renderer, 512, 384, 197, 566); // Draw x axis
  SDL_RenderDrawLine(renderer, 197, 566, 203, 557);
  SDL_RenderDrawLine(renderer, 197, 566, 205, 566);
  SDL_RenderDrawLine(renderer, 219, 559, 209, 553); // one tick Lx = 344
  SDL_RenderDrawLine(renderer, 512, 384, 827, 566); // Draw y axis
  SDL_RenderDrawLine(renderer, 827, 566, 821, 557);
  SDL_RenderDrawLine(renderer, 827, 566, 819, 566);
  SDL_RenderDrawLine(renderer, 805, 559, 815, 553); // one tick Ly = 344
  SDL_RenderPresent(renderer);
}

// Clear screen
void w_clear() {
  SDL_SetRenderDrawColor(renderer, 0, 0, 0, SDL_ALPHA_OPAQUE); // black
  SDL_RenderClear(renderer);
  SDL_RenderPresent(renderer);
}